<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 15/03/19
 * Time: 8.30
 */

namespace FiloBlu\SeoImport\Model\Import;

use FiloBlu\SeoImport\Model\Import\AdvancedImport\RowValidatorInterface as ValidatorInterface;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\ImportExport\Model\Import\ErrorProcessing\ProcessingErrorAggregatorInterface;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Store\Model\StoreManagerInterface;

class AdvancedImport extends \Magento\ImportExport\Model\Import\Entity\AbstractEntity
{


    const ID = 'id_categoria';
    const STORE_VIEW = 'store_view';
    protected $fields = [
        'id_categoria',
        'store_view',
        'name',
        'category_seo_name',
        'url_key',
        'meta_title',
        'meta_description',
        'meta_keywords',
        'meta_robots',
        'description'
    ];
    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = [
        ValidatorInterface::ERROR_ID_IS_EMPTY => 'Empty',
    ];

    protected $_permanentAttributes = [self::ID];

    /**
     * If we should check column names
     *
     * @var bool
     */
    protected $needColumnCheck = true;

    /**
     * Valid column names
     *
     * @array
     */
    protected $validColumnNames = [
        'id_categoria',
        'store_view',
        'nome_categoria',
        'seo_name_h1',
        'url_key',
        'meta_title',
        'meta_description',
        'meta_keywords',
        'meta_robots',
        'contenuto_categoria'
    ];

    /**
     * Need to log in import history
     *
     * @var bool
     */
    protected $logInHistory = true;

    protected $_validators = [];

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $_connection;

    protected $_resource;


    private $categoryRepository;
    private $storeRepository;
    private $storeManager;

    /**
     * AdvancedImport constructor.
     * @param \Magento\Framework\Json\Helper\Data $jsonHelper
     * @param \Magento\ImportExport\Helper\Data $importExportData
     * @param \Magento\ImportExport\Model\ResourceModel\Import\Data $importData
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\ImportExport\Model\ResourceModel\Helper $resourceHelper
     * @param ProcessingErrorAggregatorInterface $errorAggregator
     * @param CategoryRepositoryInterface $categoryRepository
     * @param StoreRepositoryInterface $storeRepository
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\ImportExport\Helper\Data $importExportData,
        \Magento\ImportExport\Model\ResourceModel\Import\Data $importData,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\ImportExport\Model\ResourceModel\Helper $resourceHelper,
        ProcessingErrorAggregatorInterface $errorAggregator,
        CategoryRepositoryInterface $categoryRepository,
        StoreRepositoryInterface $storeRepository,
        StoreManagerInterface $storeManager
    )
    {
        $this->jsonHelper = $jsonHelper;
        $this->_importExportData = $importExportData;
        $this->_resourceHelper = $resourceHelper;
        $this->_dataSourceModel = $importData;
        $this->_resource = $resource;
        $this->_connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->errorAggregator = $errorAggregator;

        $this->categoryRepository = $categoryRepository;
        $this->storeRepository = $storeRepository;
        $this->storeManager = $storeManager;

    }

    /**
     * @return array
     */
    public function getValidColumnNames()
    {
        return $this->validColumnNames;
    }

    /**
     * Entity type code getter.
     *
     * @return string
     */
    public function getEntityTypeCode()
    {
        return 'filoblu_seoimport_import';
    }

    /**
     * @param array $rowData
     * @param int $rowNum
     * @return bool|AdvancedImport
     */

    public function validateRow(array $rowData, $rowNum)
    {
        if ($this->getBehavior() != \Magento\ImportExport\Model\Import::BEHAVIOR_APPEND) {
            return $this->behaviourNotAllowed();
        } else {
            $title = false;
            if (isset($this->_validatedRows[$rowNum])) {
                return !$this->getErrorAggregator()->isRowInvalid($rowNum);
            }

            $this->_validatedRows[$rowNum] = true;
            return !$this->getErrorAggregator()->isRowInvalid($rowNum);
        }


    }

    /**
     * @return bool
     * @throws NoSuchEntityException
     * @throws \Magento\Framework\Exception\CouldNotSaveException
     * @throws \Magento\Store\Model\StoreIsInactiveException
     */
    protected function _importData()
    {
        if (\Magento\ImportExport\Model\Import::BEHAVIOR_DELETE == $this->getBehavior()) {
            $this->behaviourNotAllowed();
        } elseif (\Magento\ImportExport\Model\Import::BEHAVIOR_REPLACE == $this->getBehavior()) {
            $this->behaviourNotAllowed();
        } elseif (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $this->getBehavior()) {
            $this->saveAndReplaceEntity();
        }
        return true;
    }

    /**
     * @return AdvancedImport
     */
    protected function behaviourNotAllowed()
    {
        return $this->addRowError(ValidatorInterface::ERROR_OP_NOT_ALLOWED, 0);
    }

    /**
     * @return $this
     * @throws NoSuchEntityException
     * @throws \Magento\Framework\Exception\CouldNotSaveException
     * @throws \Magento\Store\Model\StoreIsInactiveException
     */
    protected function saveAndReplaceEntity()
    {

        while ($bunch = $this->_dataSourceModel->getNextBunch()) {

            foreach ($bunch as $rowNum => $rowData) {
                if (!$this->validateRow($rowData, $rowNum)) {
                    $this->addRowError(ValidatorInterface::ERROR_MESSAGE_IS_EMPTY, $rowNum);
                    continue;
                }
                if ($this->getErrorAggregator()->hasToBeTerminated()) {
                    $this->getErrorAggregator()->addRowToSkip($rowNum);
                    continue;
                }

                $row_cat_id = $rowData[self::ID];
                $row_store_view = $rowData[self::STORE_VIEW];

                foreach ($rowData as $key => $value) {
                    if ($key == self::STORE_VIEW || $key == self::ID) {

                    } else {
                        $key_index = array_search($key, $this->validColumnNames);
                        $new_key = $this->fields[$key_index];
                        $this->importSeoData($row_cat_id, $row_store_view, array($new_key => $value));
                    }

                }


            }
        }
        return $this;
    }


    /**
     * @param int $cat_id
     * @param string $store_view
     * @param array $data
     * @return string
     * @throws NoSuchEntityException
     * @throws \Magento\Framework\Exception\CouldNotSaveException
     * @throws \Magento\Store\Model\StoreIsInactiveException
     */
    public function importSeoData(int $cat_id, string $store_view, array $data)
    {

        $store_id = (int)$this->storeRepository->getActiveStoreByCode($store_view)->getId();
        $this->storeManager->setCurrentStore($store_id);
        try {
            $cat = $this->categoryRepository->get($cat_id, $store_id);

            foreach ($data as $key => $val) {
                if ($key == "url_key") {
                    $t = explode(".", $val);
                    $val = $t[0];
                    $val = strtolower(str_replace(" ", "-", $val));

                }
                $cat->setData($key, $val);
            }

            $this->categoryRepository->save($cat);
        } catch (NoSuchEntityException $error) {
            return $error->getMessage();
        }
    }


}
